`include "utran_inst_cov.sv" 
`include "umsg.sv"

class umonitor_inst extends uvm_monitor #(utran_inst);

 `uvm_component_utils(umonitor_inst)
 
  uvm_analysis_port #(utran_inst) ap;
  
  virtual instructionIF.monitor mi;
  
  bit [15:0]	inst;
 
  function new (string name, uvm_component parent);
	super.new(name, parent);
  endfunction: new

  function void build_phase(uvm_phase phase);
   super.build_phase(phase);
   ap = new("inst_group_cov_ap", this);
  endfunction: build_phase
  
  task run_phase(uvm_phase phase);
	utran_inst m_inst;
	m_inst = new();
    forever
    begin
      @(posedge mi.ram_clk) // Modified for async
	  inst = mi.inst;
		m_inst.opc1 	= opcode1'	(inst[15:12]);
		m_inst.opc3 	= opcode3'	(inst[7:4]);
		m_inst.opc3r 	= opcode3r'	(inst[7:4]);
		m_inst.opc4 	= opcode4'	(inst[3:0]);
		m_inst.cond 	= condition'(inst[11:8]);
		m_inst.Rd 		= RegD'		(inst[11:8]);
		m_inst.Rd_br	= RegD_br'	(inst[7:4]);
		m_inst.Rs1 		= RegS1'	(inst[7:4]);
		m_inst.Rs2 		= RegS2'	(inst[3:0]);
		m_inst.Uk4 		= 			inst[3:0];
		m_inst.K4		= 			inst[3:0];
		m_inst.K8		= 			inst[7:0];
		m_inst.K12 		= 			inst[11:0];
	
		case(m_inst.opc1)
		  ori4_op1:  m_inst.iType = Type_I;
		  add2_op1:  m_inst.iType = Type_I; 
		  sub2_op1:  m_inst.iType = Type_I;
		  lw_op1: 	 m_inst.iType = Type_II;
		  lb_op1: 	 m_inst.iType = Type_II; 
		  sw_op1: 	 m_inst.iType = Type_II;
		  sb_op1: 	 m_inst.iType = Type_II;
		  alu_op1: 	 m_inst.iType = Type_III; 		
		  alur1_op1: m_inst.iType = Type_IV; 
		  brpc_op1:  m_inst.iType = Type_V;
		  br_op1: 	 m_inst.iType = Type_VI;
		  //empty_op1: 	m_inst.iType = UNDEFINED;
		  add3_op1:  m_inst.iType = Type_VII;
		  sub3_op1:  m_inst.iType = Type_VII;
		  cmd_op1: 	 m_inst.iType = Type_VIII;
		  ldir1_op1: m_inst.iType = Type_IX;
		  //default: 		iType = UNDEFINED;
		endcase;
      ap.write(m_inst);
	  #1;
	end
  endtask : run_phase
  
endclass : umonitor_inst







