`include "umsg.sv"

class utran_inst extends uvm_sequence_item;
  
  // Transaction data contents
  //

  rand bit [14:0]	UnsgVal;
  
  rand instruction inst;
  rand opcode1 opc1;
  rand opcode2 opc2;
  rand opcode3 opc3;
  rand stack Rstack, Dstack;

  
  // Register field macros
  //
  `uvm_object_utils_begin(utran_inst)
    `uvm_field_enum(instruction, inst, UVM_ALL_ON) 
	`uvm_field_enum(opcode1, opc1, UVM_ALL_ON) 
	`uvm_field_enum(opcode2, opc2, UVM_ALL_ON) 
	`uvm_field_enum(opcode3, opc3, UVM_ALL_ON)
	`uvm_field_enum(stack, Rstack, UVM_ALL_ON) 
	`uvm_field_enum(stack, Dstack, UVM_ALL_ON) 
	`uvm_field_int(UnsgVal, UVM_ALL_ON)
  `uvm_object_utils_end  
  
  function new (string name = "");
    super.new(name);
  endfunction: new
  
  constraint instruction {
  
    inst == lit 			-> opc1 == lit_op1;
	inst == jmp 			-> opc1 == normal & opc2 == jmp_op2;
	inst == cond_jmp 		-> opc1 == normal & opc2 == cond_jmp_op2;
	inst == call 			-> opc1 == normal & opc2 == call_op2;
	inst == nop 			-> opc1 == normal & opc2 == alu_op2 & opc3 == nop_op3;
	inst == nip 			-> opc1 == normal & opc2 == alu_op2 & opc3 == nip_op3;
	inst == exit 			-> opc1 == normal & opc2 == alu_op2 & opc3 == exit_op3;
	inst == dup 			-> opc1 == normal & opc2 == alu_op2 & opc3 == dup_op3;
	inst == rs_push 		-> opc1 == normal & opc2 == alu_op2 & opc3 == rs_push_op3;
	inst == swap 			-> opc1 == normal & opc2 == alu_op2 & opc3 == swap_op3;
	inst == over 			-> opc1 == normal & opc2 == alu_op2 & opc3 == over_op3;
	inst == drop 			-> opc1 == normal & opc2 == alu_op2 & opc3 == drop_op3;
	inst == mem_wr 			-> opc1 == normal & opc2 == alu_op2 & opc3 == mem_wr_op3;
	inst == add_op 			-> opc1 == normal & opc2 == alu_op2 & opc3 == add_op3;
	inst == and_op 			-> opc1 == normal & opc2 == alu_op2 & opc3 == and_op3;
	inst == or_op 			-> opc1 == normal & opc2 == alu_op2 & opc3 == or_op3;
	inst == xor_op 			-> opc1 == normal & opc2 == alu_op2 & opc3 == xor_op3;
	inst == invert 			-> opc1 == normal & opc2 == alu_op2 & opc3 == invert_op3;
	inst == n_eql_t 		-> opc1 == normal & opc2 == alu_op2 & opc3 == n_eql_t_op3;
	inst == n_lt_t 			-> opc1 == normal & opc2 == alu_op2 & opc3 == n_lt_t_op3;
	inst == n_sr_t 			-> opc1 == normal & opc2 == alu_op2 & opc3 == n_sr_t_op3;
	inst == sub1 			-> opc1 == normal & opc2 == alu_op2 & opc3 == sub1_op3;
	inst == rs_pop 			-> opc1 == normal & opc2 == alu_op2 & opc3 == rs_pop_op3;
	inst == rs_cp 			-> opc1 == normal & opc2 == alu_op2 & opc3 == rs_cp_op3;
	inst == mem_rd 			-> opc1 == normal & opc2 == alu_op2 & opc3 == mem_rd_op3;
	inst == n_sl_t 			-> opc1 == normal & opc2 == alu_op2 & opc3 == n_sl_t_op3;
	inst == stk_dep 		-> opc1 == normal & opc2 == alu_op2 & opc3 == stk_dep_op3;
	inst == n_ult_t 		-> opc1 == normal & opc2 == alu_op2 & opc3 == n_ult_t_op3;
  }

endclass : utran_inst
