`include "umsg.sv"

class inst_cov_collector extends uvm_subscriber #(utran_inst);

  `uvm_component_utils(inst_cov_collector)
  
  local bit [3:0] 	Uk4, K4;
  local bit [7:0] 	K8;
  local bit [11:0] 	K12;
  
  local instruction_type iType;
  local opcode1 opc1;
  local opcode3 opc3;
  local opcode3r opc3r;
  local opcode4 opc4;
  local condition cond;

  local RegD Rd;
  local RegD_br Rd_br;
  local RegS1 Rs1;
  local RegS2 Rs2;


 //------------------------
// Instructions
    
  covergroup inst_cg;
    option.at_least = 10;
    cp_opc1: coverpoint iType;
  endgroup  
  
 //------------------------
// Instruction Type I
  
  covergroup inst_cg_type_I;
    option.at_least = 1;
	cp_opc1: coverpoint opc1
	{ 
	  bins ori4 = { ori4_op1 };
	  bins add2 = { add2_op1 };
	  bins sub2 = { sub2_op1 };
	}
	cp_rd: coverpoint Rd;
	cp_rs1: coverpoint Rs1;
	cp_uk4: coverpoint Uk4;
	cross cp_opc1, cp_rd;
	cross cp_opc1, cp_rs1;
	cross cp_rd, cp_rs1;
  endgroup

 //------------------------
// Instruction Type II
  
  covergroup inst_cg_type_II;
    option.at_least = 1;
	cp_opc1: coverpoint opc1
	{ 
	  bins lw = { lw_op1 };
	  bins lb = { lb_op1 };
	  bins sw = { sw_op1 };
	  bins sb = { sb_op1 };
	}
	cp_rd: coverpoint Rd;
	cp_rs1: coverpoint Rs1;
	cp_uk4: coverpoint K4;
	cross cp_opc1, cp_rd;
	cross cp_opc1, cp_rs1;
	cross cp_rd, cp_rs1;
  endgroup

 //------------------------
// Instruction Type III

  covergroup inst_cg_type_III;
    option.at_least = 1;
	cp_opc3: coverpoint opc3
	{ 
	  bins and_op3 		= { and_op3 };
	  bins or_op3 		= { or_op3 };
	  bins xor_op3 		= { xor_op3 };
	  bins xnor_op3 	= { xnor_op3 };
	  bins sra_op3 		= { sra_op3 };
	  bins srl_op3 		= { srl_op3 };
	  bins sll_op3 		= { sll_op3 };
	  bins sext_op3 	= { sext_op3 };
	  bins add2c_op3 	= { add2c_op3 };
	  bins sub2c_op3 	= { sub2c_op3 };
	}
	cp_rd: coverpoint Rd;
	cp_rs2: coverpoint Rs2;
	cross cp_opc3, cp_rd;
	cross cp_opc3, cp_rs2;
	cross cp_rd, cp_rs2;
  endgroup
 
 //------------------------
// Instruction Type IV
 
  covergroup inst_cg_type_IV;
    option.at_least = 1;
	cp_opc3r: coverpoint opc3r
	{ 
	  bins andr1_op3 	= { andr1_op3 };
	  bins orr1_op3 	= { orr1_op3 };
	  bins xorr1_op3 	= { xorr1_op3 };
	  bins xnorr1_op3 	= { xnorr1_op3 };
	  bins srai_op3 	= { srai_op3 };
	  bins srli_op3 	= { srli_op3 };
	  bins slli_op3 	= { slli_op3 };
	}
	cp_rd: coverpoint Rd;
	cp_uk4: coverpoint Uk4;
	cross cp_opc3r, cp_rd;
	//cross cp_opc3r, cp_uk4;
	//cross cp_rd, cp_uk4;
  endgroup

 //------------------------
// Instruction Type V
  
  covergroup inst_cg_type_V;
    option.at_least = 1;
	cp_opc1: coverpoint opc1
	{ 
	  bins brpc = { brpc_op1 };
	}
	cp_rd: coverpoint cond;
	cp_rs1: coverpoint K8;
  endgroup

 //------------------------
// Instruction Type VI
  
  covergroup inst_cg_type_VI;
    option.at_least = 1;
	cp_opc1: coverpoint opc1
	{ 
	  bins br = { br_op1 };
	}
	cp_cond: coverpoint cond;
	cp_rd_br: coverpoint Rd_br;
	cp_k4: coverpoint K4;
	cross cp_cond, cp_rd_br;
  endgroup

 //------------------------
// Instruction Type VII
  
  covergroup inst_cg_type_VII;
    option.at_least = 1;
	cp_opc1: coverpoint opc1
	{ 
	  bins add3_op1 = { add3_op1 };
	  bins sub3_op1 = { sub3_op1 };
	}
	cp_rd: coverpoint Rd;
	cp_rs1: coverpoint Rs1;
	cp_rs2: coverpoint Rs2;
	cross cp_opc1, cp_rd;
	cross cp_opc1, cp_rs1;
	cross cp_opc1, cp_rs2;
	cross cp_rd, cp_rs1;
	cross cp_rd, cp_rs2;
	cross cp_rs1, cp_rs2;
  endgroup

 //------------------------
// Instruction Type VIII
  
  covergroup inst_cg_type_VIII;
    option.at_least = 1;
	cp_opc4: coverpoint opc4
	{
	  bins mtsr_op4 = { mtsr_op4 };
	  bins mfsr_op4 = { mfsr_op4 };
	  bins ret_op4 = { ret_op4 };
	  bins halt_op4 = { halt_op4 };
	}
	cp_rd: coverpoint Rd;
	cp_rs1: coverpoint Rs1;
	cross cp_opc4, cp_rd;
	cross cp_opc4, cp_rs1;
	cross cp_rd, cp_rs1;
  endgroup

 //------------------------
// Instruction Type IX
  
  covergroup inst_cg_type_IX;
    option.at_least = 1;
	cp_opc1: coverpoint opc1
	{
	  bins ldir1 = { ldir1_op1 };
	}
	cp_rd: coverpoint K12;
  endgroup

//------------------------
  
  function new (string name, uvm_component parent);
    super.new(name, parent);
	inst_cg = new;
	inst_cg_type_I = new;
	inst_cg_type_II = new;
	inst_cg_type_III = new;
	inst_cg_type_IV = new;
	inst_cg_type_V = new;
	inst_cg_type_VI = new;
	inst_cg_type_VII = new;
	inst_cg_type_VIII = new;
	inst_cg_type_IX = new;
  endfunction: new
  
  function void write(utran_inst t);
    // Copy to class data members
	iType	= t.iType;
	opc1 	= t.opc1;
	opc3 	= t.opc3;
	opc3r	= t.opc3r;
	opc4 	= t.opc4;
	cond 	= t.cond;
	Rd 		= t.Rd;
	Rd_br 	= t.Rd_br;
	Rs1 	= t.Rs1;
	Rs2 	= t.Rs2;
	Uk4 	= t.Uk4;
	K4 		= t.K4;
	K8 		= t.K8;
	K12 	= t.K12;
	
	case(iType)
	  Type_I	: inst_cg_type_I.sample();
 	  Type_II	: inst_cg_type_II.sample();
	  Type_III	: inst_cg_type_III.sample();
	  Type_IV	: inst_cg_type_IV.sample();
	  Type_V	: inst_cg_type_V.sample();
	  Type_VI	: inst_cg_type_VI.sample();
	  Type_VII	: inst_cg_type_VII.sample();
	  Type_VIII	: inst_cg_type_VIII.sample();
	  Type_IX	: inst_cg_type_IX.sample();
	  //default	:  
	endcase
	
	inst_cg.sample();
	
  endfunction : write

endclass : inst_cov_collector
