`include "umsg.sv"

class utran_inst extends uvm_sequence_item;
  
  // Transaction data contents
  //

  rand bit [3:0] 	Uk4, K4;
  rand bit [7:0] 	K8;
  rand bit [11:0] 	K12;
  
  rand instruction_type iType;
  rand opcode1 opc1;
  rand opcode3 opc3;
  rand opcode3r opc3r;
  rand opcode4 opc4;
  rand condition cond;

  rand RegD Rd;
  rand RegD_br Rd_br;
  rand RegS1 Rs1;
  rand RegS2 Rs2;
  
  // Register field macros
  //
  `uvm_object_utils_begin(utran_inst)
    `uvm_field_enum(instruction_type, iType, UVM_ALL_ON) 
	`uvm_field_enum(opcode1, opc1, UVM_ALL_ON) 
	`uvm_field_enum(opcode3, opc3, UVM_ALL_ON) 
	`uvm_field_enum(opcode3r, opc3r, UVM_ALL_ON) 
	`uvm_field_enum(opcode4, opc4, UVM_ALL_ON) 
	`uvm_field_enum(condition, cond, UVM_ALL_ON) 
	`uvm_field_enum(RegD, Rd, UVM_ALL_ON) 
	`uvm_field_enum(RegD_br, Rd_br, UVM_ALL_ON)
	`uvm_field_enum(RegS1, Rs1, UVM_ALL_ON)
	`uvm_field_enum(RegS2, Rs2, UVM_ALL_ON)
	`uvm_field_int(Uk4, UVM_ALL_ON)
	`uvm_field_int(K4, UVM_ALL_ON)
	`uvm_field_int(K8, UVM_ALL_ON)
	`uvm_field_int(K12, UVM_ALL_ON)
  `uvm_object_utils_end  
  
  function new (string name = "");
    super.new(name);
  endfunction: new

  constraint idist{
	iType dist { 	Type_I		:= 3,
					Type_II		:= 4,
					Type_III	:= 10,
					Type_IV		:= 8,
					Type_V		:= 1,
					Type_VI		:= 1,
					Type_VII	:= 2,
					Type_VIII	:= 4,
					Type_IX		:= 1
				};
  }
  
  constraint insttype {
  
    iType == Type_I -> opc1 inside {ori4_op1, add2_op1, sub2_op1};
	iType == Type_II -> opc1 inside {lw_op1, lb_op1, sw_op1, sb_op1};
	iType == Type_III -> opc1 inside {alu_op1};
	iType == Type_IV -> opc1 inside {alur1_op1};
	iType == Type_V -> opc1 inside {brpc_op1};
	iType == Type_VI -> opc1 inside {br_op1};
	iType == Type_VII -> opc1 inside {add3_op1, sub3_op1};
	iType == Type_VIII -> opc1 inside {cmd_op1};
	iType == Type_IX -> opc1 inside {ldir1_op1};
  }
  
  constraint regsize {
    Rd < 15; Rd > 0;
	Rd_br < 15; Rd_br > 0;
	Rs1 < 15; Rs1 > 0;
	Rs2 < 15; Rs1 > 0;
  }
  
  constraint emptyop {
    opc3r != empty_op3;
  }
  


endclass : utran_inst
